'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var g = require('@antv/g');
var gPluginWebglRenderer = require('@antv/g-plugin-webgl-renderer');

var NAMED_TAG = "named";
var INJECT_TAG = "inject";
var TAGGED = "inversify:tagged";
var TAGGED_PROP = "inversify:tagged_props";
var PARAM_TYPES = "inversify:paramtypes";
var DESIGN_PARAM_TYPES = "design:paramtypes";

var idCounter = 0;
function id() {
    return idCounter++;
}

var DUPLICATED_INJECTABLE_DECORATOR = "Cannot apply @injectable decorator multiple times.";
var DUPLICATED_METADATA = "Metadata key was used more than once in a parameter:";
var UNDEFINED_INJECT_ANNOTATION = function (name) {
    return "@inject called with undefined this could mean that the class " + name + " has " +
        "a circular dependency problem. You can use a LazyServiceIdentifer to  " +
        "overcome this limitation.";
};
var INVALID_DECORATOR_OPERATION = "The @inject @multiInject @tagged and @named decorators " +
    "must be applied to the parameters of a class constructor or a class property.";

var Metadata = (function () {
    function Metadata(key, value) {
        this.key = key;
        this.value = value;
    }
    Metadata.prototype.toString = function () {
        if (this.key === NAMED_TAG) {
            return "named: " + this.value.toString() + " ";
        }
        else {
            return "tagged: { key:" + this.key.toString() + ", value: " + this.value + " }";
        }
    };
    return Metadata;
}());

function tagParameter(annotationTarget, propertyName, parameterIndex, metadata) {
    var metadataKey = TAGGED;
    _tagParameterOrProperty(metadataKey, annotationTarget, propertyName, metadata, parameterIndex);
}
function tagProperty(annotationTarget, propertyName, metadata) {
    var metadataKey = TAGGED_PROP;
    _tagParameterOrProperty(metadataKey, annotationTarget.constructor, propertyName, metadata);
}
function _tagParameterOrProperty(metadataKey, annotationTarget, propertyName, metadata, parameterIndex) {
    var paramsOrPropertiesMetadata = {};
    var isParameterDecorator = (typeof parameterIndex === "number");
    var key = (parameterIndex !== undefined && isParameterDecorator) ? parameterIndex.toString() : propertyName;
    if (isParameterDecorator && propertyName !== undefined) {
        throw new Error(INVALID_DECORATOR_OPERATION);
    }
    if (Reflect.hasOwnMetadata(metadataKey, annotationTarget)) {
        paramsOrPropertiesMetadata = Reflect.getMetadata(metadataKey, annotationTarget);
    }
    var paramOrPropertyMetadata = paramsOrPropertiesMetadata[key];
    if (!Array.isArray(paramOrPropertyMetadata)) {
        paramOrPropertyMetadata = [];
    }
    else {
        for (var _i = 0, paramOrPropertyMetadata_1 = paramOrPropertyMetadata; _i < paramOrPropertyMetadata_1.length; _i++) {
            var m = paramOrPropertyMetadata_1[_i];
            if (m.key === metadata.key) {
                throw new Error(DUPLICATED_METADATA + " " + m.key.toString());
            }
        }
    }
    paramOrPropertyMetadata.push(metadata);
    paramsOrPropertiesMetadata[key] = paramOrPropertyMetadata;
    Reflect.defineMetadata(metadataKey, paramsOrPropertiesMetadata, annotationTarget);
}

function inject(serviceIdentifier) {
    return function (target, targetKey, index) {
        if (serviceIdentifier === undefined) {
            throw new Error(UNDEFINED_INJECT_ANNOTATION(target.name));
        }
        var metadata = new Metadata(INJECT_TAG, serviceIdentifier);
        if (typeof index === "number") {
            tagParameter(target, targetKey, index, metadata);
        }
        else {
            tagProperty(target, targetKey, metadata);
        }
    };
}

var __spreadArray = (undefined && undefined.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};

var __spreadArray$1 = (undefined && undefined.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};

var __awaiter = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __spreadArray$2 = (undefined && undefined.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};

var ContainerModule = (function () {
    function ContainerModule(registry) {
        this.id = id();
        this.registry = registry;
    }
    return ContainerModule;
}());

function injectable() {
    return function (target) {
        if (Reflect.hasOwnMetadata(PARAM_TYPES, target)) {
            throw new Error(DUPLICATED_INJECTABLE_DECORATOR);
        }
        var types = Reflect.getMetadata(DESIGN_PARAM_TYPES, target) || [];
        Reflect.defineMetadata(PARAM_TYPES, types, target);
        return target;
    };
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);

    if (enumerableOnly) {
      symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
    }

    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === "object" || typeof call === "function")) {
    return call;
  }

  return _assertThisInitialized(self);
}

function _createSuper(Derived) {
  var hasNativeReflectConstruct = _isNativeReflectConstruct();

  return function _createSuperInternal() {
    var Super = _getPrototypeOf(Derived),
        result;

    if (hasNativeReflectConstruct) {
      var NewTarget = _getPrototypeOf(this).constructor;

      result = Reflect.construct(Super, arguments, NewTarget);
    } else {
      result = Super.apply(this, arguments);
    }

    return _possibleConstructorReturn(this, result);
  };
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __decorate(decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
}

function __metadata(metadataKey, metadataValue) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(metadataKey, metadataValue);
}

/**
 * Common utilities
 * @module glMatrix
 */
var ARRAY_TYPE = typeof Float32Array !== 'undefined' ? Float32Array : Array;
if (!Math.hypot) Math.hypot = function () {
  var y = 0,
      i = arguments.length;

  while (i--) {
    y += arguments[i] * arguments[i];
  }

  return Math.sqrt(y);
};

/**
 * 3x3 Matrix
 * @module mat3
 */

/**
 * Creates a new identity mat3
 *
 * @returns {mat3} a new 3x3 matrix
 */

function create() {
  var out = new ARRAY_TYPE(9);

  if (ARRAY_TYPE != Float32Array) {
    out[1] = 0;
    out[2] = 0;
    out[3] = 0;
    out[5] = 0;
    out[6] = 0;
    out[7] = 0;
  }

  out[0] = 1;
  out[4] = 1;
  out[8] = 1;
  return out;
}

/**
 * 3 Dimensional Vector
 * @module vec3
 */

/**
 * Creates a new, empty vec3
 *
 * @returns {vec3} a new 3D vector
 */

function create$1() {
  var out = new ARRAY_TYPE(3);

  if (ARRAY_TYPE != Float32Array) {
    out[0] = 0;
    out[1] = 0;
    out[2] = 0;
  }

  return out;
}
/**
 * Creates a new vec3 initialized with the given values
 *
 * @param {Number} x X component
 * @param {Number} y Y component
 * @param {Number} z Z component
 * @returns {vec3} a new 3D vector
 */

function fromValues(x, y, z) {
  var out = new ARRAY_TYPE(3);
  out[0] = x;
  out[1] = y;
  out[2] = z;
  return out;
}
/**
 * Adds two vec3's
 *
 * @param {vec3} out the receiving vector
 * @param {ReadonlyVec3} a the first operand
 * @param {ReadonlyVec3} b the second operand
 * @returns {vec3} out
 */

function add(out, a, b) {
  out[0] = a[0] + b[0];
  out[1] = a[1] + b[1];
  out[2] = a[2] + b[2];
  return out;
}
/**
 * Subtracts vector b from vector a
 *
 * @param {vec3} out the receiving vector
 * @param {ReadonlyVec3} a the first operand
 * @param {ReadonlyVec3} b the second operand
 * @returns {vec3} out
 */

function subtract(out, a, b) {
  out[0] = a[0] - b[0];
  out[1] = a[1] - b[1];
  out[2] = a[2] - b[2];
  return out;
}
/**
 * Performs a linear interpolation between two vec3's
 *
 * @param {vec3} out the receiving vector
 * @param {ReadonlyVec3} a the first operand
 * @param {ReadonlyVec3} b the second operand
 * @param {Number} t interpolation amount, in the range [0-1], between the two inputs
 * @returns {vec3} out
 */

function lerp(out, a, b, t) {
  var ax = a[0];
  var ay = a[1];
  var az = a[2];
  out[0] = ax + t * (b[0] - ax);
  out[1] = ay + t * (b[1] - ay);
  out[2] = az + t * (b[2] - az);
  return out;
}
/**
 * Alias for {@link vec3.subtract}
 * @function
 */

var sub = subtract;
/**
 * Perform some operation over an array of vec3s.
 *
 * @param {Array} a the array of vectors to iterate over
 * @param {Number} stride Number of elements between the start of each vec3. If 0 assumes tightly packed
 * @param {Number} offset Number of elements to skip at the beginning of the array
 * @param {Number} count Number of vec3s to iterate over. If 0 iterates over entire array
 * @param {Function} fn Function to call for each vector in the array
 * @param {Object} [arg] additional argument to pass to fn
 * @returns {Array} a
 * @function
 */

var forEach = function () {
  var vec = create$1();
  return function (a, stride, offset, count, fn, arg) {
    var i, l;

    if (!stride) {
      stride = 3;
    }

    if (!offset) {
      offset = 0;
    }

    if (count) {
      l = Math.min(count * stride + offset, a.length);
    } else {
      l = a.length;
    }

    for (i = offset; i < l; i += stride) {
      vec[0] = a[i];
      vec[1] = a[i + 1];
      vec[2] = a[i + 2];
      fn(vec, vec, arg);
      a[i] = vec[0];
      a[i + 1] = vec[1];
      a[i + 2] = vec[2];
    }

    return a;
  };
}();

var CubeUpdater = /*#__PURE__*/function () {
  function CubeUpdater() {
    _classCallCheck(this, CubeUpdater);

    this.dependencies = ['height', 'width', 'depth', 'anchor'];
  }

  _createClass(CubeUpdater, [{
    key: "update",
    value: function update(attributes, aabb) {
      var _attributes$height = attributes.height,
          height = _attributes$height === void 0 ? 0 : _attributes$height,
          _attributes$width = attributes.width,
          width = _attributes$width === void 0 ? 0 : _attributes$width,
          _attributes$depth = attributes.depth,
          depth = _attributes$depth === void 0 ? 0 : _attributes$depth,
          _attributes$anchor = attributes.anchor,
          anchor = _attributes$anchor === void 0 ? [0, 0] : _attributes$anchor; // anchor is left-top by default
      // attributes.x = minX + anchor[0] * width;
      // attributes.y = minY + anchor[1] * height;

      var halfExtents = fromValues(width / 2, height / 2, depth / 2);
      var center = fromValues((1 - anchor[0] * 2) * halfExtents[0], (1 - anchor[1] * 2) * halfExtents[1], (1 - anchor[1] * 2) * halfExtents[2]);
      aabb.update(center, halfExtents);
    }
  }]);

  return CubeUpdater;
}();

CubeUpdater = __decorate([injectable()], CubeUpdater);

var SphereUpdater = /*#__PURE__*/function () {
  function SphereUpdater() {
    _classCallCheck(this, SphereUpdater);

    this.dependencies = ['height', 'width', 'depth', 'anchor'];
  }

  _createClass(SphereUpdater, [{
    key: "update",
    value: function update(attributes, aabb) {
      var _attributes$height = attributes.height,
          height = _attributes$height === void 0 ? 0 : _attributes$height,
          _attributes$width = attributes.width,
          width = _attributes$width === void 0 ? 0 : _attributes$width,
          _attributes$depth = attributes.depth,
          depth = _attributes$depth === void 0 ? 0 : _attributes$depth,
          _attributes$anchor = attributes.anchor,
          anchor = _attributes$anchor === void 0 ? [0, 0] : _attributes$anchor; // anchor is left-top by default
      // attributes.x = minX + anchor[0] * width;
      // attributes.y = minY + anchor[1] * height;

      var halfExtents = fromValues(width / 2, height / 2, depth / 2);
      var center = fromValues((1 - anchor[0] * 2) * halfExtents[0], (1 - anchor[1] * 2) * halfExtents[1], (1 - anchor[1] * 2) * halfExtents[2]);
      aabb.update(center, halfExtents);
    }
  }]);

  return SphereUpdater;
}();

SphereUpdater = __decorate([injectable()], SphereUpdater);

var GridUpdater = /*#__PURE__*/function () {
  function GridUpdater() {
    _classCallCheck(this, GridUpdater);

    this.dependencies = ['width', 'height', 'anchor'];
  }

  _createClass(GridUpdater, [{
    key: "update",
    value: function update(attributes, aabb) {
      var _attributes$width = attributes.width,
          width = _attributes$width === void 0 ? 0 : _attributes$width,
          _attributes$height = attributes.height,
          height = _attributes$height === void 0 ? 0 : _attributes$height,
          _attributes$anchor = attributes.anchor,
          anchor = _attributes$anchor === void 0 ? [0, 0] : _attributes$anchor;
      var halfExtents = fromValues(width / 2, height / 2, 1);
      var center = fromValues((1 - anchor[0] * 2) * halfExtents[0], (1 - anchor[1] * 2) * halfExtents[1], (1 - anchor[1] * 2) * halfExtents[2]);
      aabb.update(center, halfExtents);
    }
  }]);

  return GridUpdater;
}();

GridUpdater = __decorate([injectable()], GridUpdater);

var imageVertex = "attribute vec3 a_Position;\nattribute vec3 a_Normal;\n\nuniform mat4 u_ProjectionMatrix;\nuniform mat4 u_ViewMatrix;\nuniform vec4 u_Color;\n\nvarying vec4 v_Color;\n\n#pragma include \"uv.declaration\"\n\n#pragma include \"instancing.declaration\"\n#pragma include \"picking\"\n\nvoid main() {\n  v_Color = u_Color;\n\n  #pragma include \"instancing\"\n\n  gl_Position = u_ProjectionMatrix * u_ViewMatrix * modelMatrix * vec4(a_Position, 1.0);\n\n  #pragma include \"uv\"\n\n  setPickingColor(a_PickingColor);\n}";

var imageFragment = "uniform float u_Opacity : 1;\n\nvarying vec4 v_Color;\n\n#pragma include \"uv.declaration\"\n#pragma include \"map.declaration\"\n\nvoid main() {\n  vec4 diffuseColor = v_Color;\n\n  #pragma include \"map\"\n\n  gl_FragColor = diffuseColor;\n  gl_FragColor.a = gl_FragColor.a * u_Opacity;\n}";

var primitiveUv1Padding = 4.0 / 64;
var primitiveUv1PaddingScale = 1.0 - primitiveUv1Padding * 2;
var ATTRIBUTE = {
  Position: 'a_Position',
  Normal: 'a_Normal',
  Uv: 'a_Uv'
};
var UNIFORM = {
  Color: 'u_Color',
  Map: 'u_Map',
  UvTransform: 'u_UvTransform'
};

var CubeModelBuilder = /*#__PURE__*/function () {
  function CubeModelBuilder() {
    _classCallCheck(this, CubeModelBuilder);

    this.shaderModule = void 0;
    this.texturePool = void 0;
  }

  _createClass(CubeModelBuilder, [{
    key: "onAttributeChanged",
    value: function () {
      var _onAttributeChanged = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(object, name, value) {
        var entity, renderable, renderable3d, material, geometry, fillColor;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                entity = object.getEntity();
                renderable = entity.getComponent(g.Renderable);
                renderable3d = entity.getComponent(gPluginWebglRenderer.Renderable3D);
                material = entity.getComponent(gPluginWebglRenderer.Material3D);
                geometry = entity.getComponent(gPluginWebglRenderer.Geometry3D);

                if (name === 'fill') {
                  fillColor = gPluginWebglRenderer.rgb2arr(value);
                  material.setUniform(_defineProperty({}, UNIFORM.Color, fillColor));
                }

              case 6:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      function onAttributeChanged(_x, _x2, _x3) {
        return _onAttributeChanged.apply(this, arguments);
      }

      return onAttributeChanged;
    }()
  }, {
    key: "prepareModel",
    value: function () {
      var _prepareModel = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(object) {
        var entity, renderable, material, geometry, renderable3d, _object$attributes, _object$attributes$fi, fill, map, _this$shaderModule$ge, vs, fs, extractedUniforms, fillColor, _material$setUniform2, texture, _this$buildAttributes, indices, positions, normals, uvs;

        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                entity = object.getEntity();
                renderable = entity.getComponent(g.Renderable);
                material = entity.getComponent(gPluginWebglRenderer.Material3D);
                geometry = entity.getComponent(gPluginWebglRenderer.Geometry3D);
                renderable3d = entity.getComponent(gPluginWebglRenderer.Renderable3D);
                _object$attributes = object.attributes, _object$attributes$fi = _object$attributes.fill, fill = _object$attributes$fi === void 0 ? '#FFF' : _object$attributes$fi, map = _object$attributes.map;
                this.shaderModule.registerModule('material-basic', {
                  vs: imageVertex,
                  fs: imageFragment
                });
                _this$shaderModule$ge = this.shaderModule.getModule('material-basic'), vs = _this$shaderModule$ge.vs, fs = _this$shaderModule$ge.fs, extractedUniforms = _this$shaderModule$ge.uniforms;
                material.vertexShaderGLSL = vs || '';
                material.fragmentShaderGLSL = fs || '';
                material.cull = {
                  enable: true,
                  face: gPluginWebglRenderer.gl.BACK
                };
                material.depth = {
                  enable: true
                };
                fillColor = gPluginWebglRenderer.rgb2arr(fill);
                material.setUniform(_objectSpread2(_objectSpread2({}, extractedUniforms), {}, _defineProperty({}, UNIFORM.Color, fillColor)));

                if (!map) {
                  _context2.next = 20;
                  break;
                }

                _context2.next = 17;
                return this.loadImage(map, 200, 200, renderable3d.engine);

              case 17:
                texture = _context2.sent;
                material.setDefines({
                  USE_UV: 1,
                  USE_MAP: 1
                });
                material.setUniform((_material$setUniform2 = {}, _defineProperty(_material$setUniform2, UNIFORM.Map, texture), _defineProperty(_material$setUniform2, UNIFORM.UvTransform, create()), _material$setUniform2));

              case 20:
                _this$buildAttributes = this.buildAttributes(object.attributes), indices = _this$buildAttributes.indices, positions = _this$buildAttributes.positions, normals = _this$buildAttributes.normals, uvs = _this$buildAttributes.uvs;
                geometry.vertexCount = indices.length;
                geometry.setIndex(indices);
                geometry.setAttribute(ATTRIBUTE.Position, Float32Array.from(positions), {
                  arrayStride: 4 * 3,
                  stepMode: 'vertex',
                  attributes: [{
                    shaderLocation: 0,
                    offset: 0,
                    format: 'float3'
                  }]
                });
                geometry.setAttribute(ATTRIBUTE.Normal, Float32Array.from(normals), {
                  arrayStride: 4 * 3,
                  stepMode: 'vertex',
                  attributes: [{
                    shaderLocation: 1,
                    offset: 0,
                    format: 'float3'
                  }]
                });
                geometry.setAttribute(ATTRIBUTE.Uv, Float32Array.from(uvs), {
                  arrayStride: 4 * 2,
                  stepMode: 'vertex',
                  attributes: [{
                    shaderLocation: 2,
                    offset: 0,
                    format: 'float2'
                  }]
                });
                renderable3d.modelPrepared = true;
                renderable.dirty = true;

              case 28:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function prepareModel(_x4) {
        return _prepareModel.apply(this, arguments);
      }

      return prepareModel;
    }()
  }, {
    key: "buildAttributes",
    value: function buildAttributes(attributes) {
      var _attributes$widthSegm = attributes.widthSegments,
          widthSegments = _attributes$widthSegm === void 0 ? 1 : _attributes$widthSegm,
          _attributes$heightSeg = attributes.heightSegments,
          heightSegments = _attributes$heightSeg === void 0 ? 1 : _attributes$heightSeg,
          _attributes$depthSegm = attributes.depthSegments,
          depthSegments = _attributes$depthSegm === void 0 ? 1 : _attributes$depthSegm,
          _attributes$height = attributes.height,
          height = _attributes$height === void 0 ? 0 : _attributes$height,
          _attributes$width = attributes.width,
          width = _attributes$width === void 0 ? 0 : _attributes$width,
          _attributes$depth = attributes.depth,
          depth = _attributes$depth === void 0 ? 0 : _attributes$depth;
      var ws = widthSegments;
      var hs = heightSegments;
      var ds = depthSegments;
      var hex = width / 2;
      var hey = height / 2;
      var hez = depth / 2;
      var corners = [fromValues(-hex, -hey, hez), fromValues(hex, -hey, hez), fromValues(hex, hey, hez), fromValues(-hex, hey, hez), fromValues(hex, -hey, -hez), fromValues(-hex, -hey, -hez), fromValues(-hex, hey, -hez), fromValues(hex, hey, -hez)];
      var faceAxes = [[0, 3, 1], [4, 7, 5], [1, 4, 0], [3, 6, 2], [1, 2, 4], [5, 6, 0] // LEFT
      ];
      var faceNormals = [[0, 0, 1], [0, 0, -1], [0, -1, 0], [0, 1, 0], [1, 0, 0], [-1, 0, 0] // LEFT
      ];
      var sides = {
        FRONT: 0,
        BACK: 1,
        TOP: 2,
        BOTTOM: 3,
        RIGHT: 4,
        LEFT: 5
      };
      var positions = [];
      var normals = [];
      var uvs = [];
      var indices = [];
      var vcounter = 0;

      var generateFace = function generateFace(side, uSegments, vSegments) {
        var u;
        var v;
        var i;
        var j;

        for (i = 0; i <= uSegments; i++) {
          for (j = 0; j <= vSegments; j++) {
            var temp1 = create$1();
            var temp2 = create$1();
            var temp3 = create$1();
            var r = create$1();
            lerp(temp1, corners[faceAxes[side][0]], corners[faceAxes[side][1]], i / uSegments);
            lerp(temp2, corners[faceAxes[side][0]], corners[faceAxes[side][2]], j / vSegments);
            sub(temp3, temp2, corners[faceAxes[side][0]]);
            add(r, temp1, temp3);
            u = i / uSegments;
            v = j / vSegments;
            positions.push(r[0], r[1], r[2]);
            normals.push(faceNormals[side][0], faceNormals[side][1], faceNormals[side][2]);
            uvs.push(u, v); // pack as 3x2
            // 1/3 will be empty, but it's either that or stretched pixels
            // TODO: generate non-rectangular lightMaps, so we could use space without stretching

            u /= 3;
            v /= 3;
            u = u * primitiveUv1PaddingScale + primitiveUv1Padding;
            v = v * primitiveUv1PaddingScale + primitiveUv1Padding;
            u += side % 3 / 3;
            v += Math.floor(side / 3) / 3;

            if (i < uSegments && j < vSegments) {
              indices.push(vcounter + vSegments + 1, vcounter + 1, vcounter);
              indices.push(vcounter + vSegments + 1, vcounter + vSegments + 2, vcounter + 1);
            }

            vcounter++;
          }
        }
      };

      generateFace(sides.FRONT, ws, hs);
      generateFace(sides.BACK, ws, hs);
      generateFace(sides.TOP, ws, ds);
      generateFace(sides.BOTTOM, ws, ds);
      generateFace(sides.RIGHT, ds, hs);
      generateFace(sides.LEFT, ds, hs);
      return {
        indices: indices,
        positions: positions,
        normals: normals,
        uvs: uvs,
        vertexCount: vcounter
      }; // TODO: barycentric & tangent
    }
  }, {
    key: "loadImage",
    value: function () {
      var _loadImage = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(img) {
        var width,
            height,
            engine,
            texture,
            _args3 = arguments;
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                width = _args3.length > 1 && _args3[1] !== undefined ? _args3[1] : 0;
                height = _args3.length > 2 && _args3[2] !== undefined ? _args3[2] : 0;
                engine = _args3.length > 3 ? _args3[3] : undefined;
                _context3.next = 5;
                return this.texturePool.getOrCreateTexture2D(engine, img, {
                  width: width,
                  height: height
                });

              case 5:
                texture = _context3.sent;
                return _context3.abrupt("return", texture);

              case 7:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function loadImage(_x5) {
        return _loadImage.apply(this, arguments);
      }

      return loadImage;
    }()
  }]);

  return CubeModelBuilder;
}();

__decorate([inject(gPluginWebglRenderer.ShaderModuleService), __metadata("design:type", Object)], CubeModelBuilder.prototype, "shaderModule", void 0);

__decorate([inject(gPluginWebglRenderer.TexturePool), __metadata("design:type", gPluginWebglRenderer.TexturePool)], CubeModelBuilder.prototype, "texturePool", void 0);

CubeModelBuilder = __decorate([injectable()], CubeModelBuilder);

var primitiveUv1Padding$1 = 4.0 / 64;
var primitiveUv1PaddingScale$1 = 1.0 - primitiveUv1Padding$1 * 2;
var ATTRIBUTE$1 = {
  Position: 'a_Position',
  Normal: 'a_Normal',
  Uv: 'a_Uv'
};
var UNIFORM$1 = {
  Color: 'u_Color',
  Map: 'u_Map',
  UvTransform: 'u_UvTransform'
};
/**
 * Render image with texture2d
 */

var SphereModelBuilder = /*#__PURE__*/function () {
  function SphereModelBuilder() {
    _classCallCheck(this, SphereModelBuilder);

    this.shaderModule = void 0;
    this.texturePool = void 0;
  }

  _createClass(SphereModelBuilder, [{
    key: "onAttributeChanged",
    value: function () {
      var _onAttributeChanged = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(object, name, value) {
        var entity, renderable, renderable3d, material, geometry, fillColor;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                entity = object.getEntity();
                renderable = entity.getComponent(g.Renderable);
                renderable3d = entity.getComponent(gPluginWebglRenderer.Renderable3D);
                material = entity.getComponent(gPluginWebglRenderer.Material3D);
                geometry = entity.getComponent(gPluginWebglRenderer.Geometry3D);

                if (name === 'fill') {
                  fillColor = gPluginWebglRenderer.rgb2arr(value);
                  material.setUniform(_defineProperty({}, UNIFORM$1.Color, fillColor));
                }

              case 6:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      function onAttributeChanged(_x, _x2, _x3) {
        return _onAttributeChanged.apply(this, arguments);
      }

      return onAttributeChanged;
    }()
  }, {
    key: "prepareModel",
    value: function () {
      var _prepareModel = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(object) {
        var entity, renderable, material, geometry, renderable3d, _object$attributes, _object$attributes$fi, fill, map, _this$shaderModule$ge, vs, fs, extractedUniforms, fillColor, _material$setUniform2, texture, _this$buildAttributes, indices, positions, normals, uvs;

        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                entity = object.getEntity();
                renderable = entity.getComponent(g.Renderable);
                material = entity.getComponent(gPluginWebglRenderer.Material3D);
                geometry = entity.getComponent(gPluginWebglRenderer.Geometry3D);
                renderable3d = entity.getComponent(gPluginWebglRenderer.Renderable3D);
                _object$attributes = object.attributes, _object$attributes$fi = _object$attributes.fill, fill = _object$attributes$fi === void 0 ? '#FFF' : _object$attributes$fi, map = _object$attributes.map;
                this.shaderModule.registerModule('material-basic', {
                  vs: imageVertex,
                  fs: imageFragment
                });
                _this$shaderModule$ge = this.shaderModule.getModule('material-basic'), vs = _this$shaderModule$ge.vs, fs = _this$shaderModule$ge.fs, extractedUniforms = _this$shaderModule$ge.uniforms;
                material.vertexShaderGLSL = vs || '';
                material.fragmentShaderGLSL = fs || '';
                material.cull = {
                  enable: true,
                  face: gPluginWebglRenderer.gl.BACK
                };
                material.depth = {
                  enable: true
                };
                fillColor = gPluginWebglRenderer.rgb2arr(fill);
                material.setUniform(_objectSpread2(_objectSpread2({}, extractedUniforms), {}, _defineProperty({}, UNIFORM$1.Color, fillColor)));

                if (!map) {
                  _context2.next = 20;
                  break;
                }

                _context2.next = 17;
                return this.loadImage(map, 200, 200, renderable3d.engine);

              case 17:
                texture = _context2.sent;
                material.setDefines({
                  USE_UV: 1,
                  USE_MAP: 1
                });
                material.setUniform((_material$setUniform2 = {}, _defineProperty(_material$setUniform2, UNIFORM$1.Map, texture), _defineProperty(_material$setUniform2, UNIFORM$1.UvTransform, create()), _material$setUniform2));

              case 20:
                _this$buildAttributes = this.buildAttributes(object.attributes), indices = _this$buildAttributes.indices, positions = _this$buildAttributes.positions, normals = _this$buildAttributes.normals, uvs = _this$buildAttributes.uvs;
                geometry.vertexCount = indices.length;
                geometry.setIndex(indices);
                geometry.setAttribute(ATTRIBUTE$1.Position, Float32Array.from(positions), {
                  arrayStride: 4 * 3,
                  stepMode: 'vertex',
                  attributes: [{
                    shaderLocation: 0,
                    offset: 0,
                    format: 'float3'
                  }]
                });
                geometry.setAttribute(ATTRIBUTE$1.Normal, Float32Array.from(normals), {
                  arrayStride: 4 * 3,
                  stepMode: 'vertex',
                  attributes: [{
                    shaderLocation: 1,
                    offset: 0,
                    format: 'float3'
                  }]
                });
                geometry.setAttribute(ATTRIBUTE$1.Uv, Float32Array.from(uvs), {
                  arrayStride: 4 * 2,
                  stepMode: 'vertex',
                  attributes: [{
                    shaderLocation: 2,
                    offset: 0,
                    format: 'float2'
                  }]
                });
                renderable3d.modelPrepared = true;
                renderable.dirty = true;

              case 28:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function prepareModel(_x4) {
        return _prepareModel.apply(this, arguments);
      }

      return prepareModel;
    }()
  }, {
    key: "buildAttributes",
    value: function buildAttributes(attributes) {
      var _attributes$widthSegm = attributes.widthSegments,
          widthSegments = _attributes$widthSegm === void 0 ? 1 : _attributes$widthSegm,
          _attributes$heightSeg = attributes.heightSegments,
          heightSegments = _attributes$heightSeg === void 0 ? 1 : _attributes$heightSeg,
          _attributes$depthSegm = attributes.depthSegments,
          depthSegments = _attributes$depthSegm === void 0 ? 1 : _attributes$depthSegm,
          _attributes$height = attributes.height,
          height = _attributes$height === void 0 ? 0 : _attributes$height,
          _attributes$width = attributes.width,
          width = _attributes$width === void 0 ? 0 : _attributes$width,
          _attributes$depth = attributes.depth,
          depth = _attributes$depth === void 0 ? 0 : _attributes$depth;
      var ws = widthSegments;
      var hs = heightSegments;
      var ds = depthSegments;
      var hex = width / 2;
      var hey = height / 2;
      var hez = depth / 2;
      var corners = [fromValues(-hex, -hey, hez), fromValues(hex, -hey, hez), fromValues(hex, hey, hez), fromValues(-hex, hey, hez), fromValues(hex, -hey, -hez), fromValues(-hex, -hey, -hez), fromValues(-hex, hey, -hez), fromValues(hex, hey, -hez)];
      var faceAxes = [[0, 3, 1], [4, 7, 5], [1, 4, 0], [3, 6, 2], [1, 2, 4], [5, 6, 0] // LEFT
      ];
      var faceNormals = [[0, 0, 1], [0, 0, -1], [0, -1, 0], [0, 1, 0], [1, 0, 0], [-1, 0, 0] // LEFT
      ];
      var sides = {
        FRONT: 0,
        BACK: 1,
        TOP: 2,
        BOTTOM: 3,
        RIGHT: 4,
        LEFT: 5
      };
      var positions = [];
      var normals = [];
      var uvs = [];
      var indices = [];
      var vcounter = 0;

      var generateFace = function generateFace(side, uSegments, vSegments) {
        var u;
        var v;
        var i;
        var j;

        for (i = 0; i <= uSegments; i++) {
          for (j = 0; j <= vSegments; j++) {
            var temp1 = create$1();
            var temp2 = create$1();
            var temp3 = create$1();
            var r = create$1();
            lerp(temp1, corners[faceAxes[side][0]], corners[faceAxes[side][1]], i / uSegments);
            lerp(temp2, corners[faceAxes[side][0]], corners[faceAxes[side][2]], j / vSegments);
            sub(temp3, temp2, corners[faceAxes[side][0]]);
            add(r, temp1, temp3);
            u = i / uSegments;
            v = j / vSegments;
            positions.push(r[0], r[1], r[2]);
            normals.push(faceNormals[side][0], faceNormals[side][1], faceNormals[side][2]);
            uvs.push(u, v); // pack as 3x2
            // 1/3 will be empty, but it's either that or stretched pixels
            // TODO: generate non-rectangular lightMaps, so we could use space without stretching

            u /= 3;
            v /= 3;
            u = u * primitiveUv1PaddingScale$1 + primitiveUv1Padding$1;
            v = v * primitiveUv1PaddingScale$1 + primitiveUv1Padding$1;
            u += side % 3 / 3;
            v += Math.floor(side / 3) / 3;

            if (i < uSegments && j < vSegments) {
              indices.push(vcounter + vSegments + 1, vcounter + 1, vcounter);
              indices.push(vcounter + vSegments + 1, vcounter + vSegments + 2, vcounter + 1);
            }

            vcounter++;
          }
        }
      };

      generateFace(sides.FRONT, ws, hs);
      generateFace(sides.BACK, ws, hs);
      generateFace(sides.TOP, ws, ds);
      generateFace(sides.BOTTOM, ws, ds);
      generateFace(sides.RIGHT, ds, hs);
      generateFace(sides.LEFT, ds, hs);
      return {
        indices: indices,
        positions: positions,
        normals: normals,
        uvs: uvs,
        vertexCount: vcounter
      }; // TODO: barycentric & tangent
    }
  }, {
    key: "loadImage",
    value: function () {
      var _loadImage = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(img) {
        var width,
            height,
            engine,
            texture,
            _args3 = arguments;
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                width = _args3.length > 1 && _args3[1] !== undefined ? _args3[1] : 0;
                height = _args3.length > 2 && _args3[2] !== undefined ? _args3[2] : 0;
                engine = _args3.length > 3 ? _args3[3] : undefined;
                _context3.next = 5;
                return this.texturePool.getOrCreateTexture2D(engine, img, {
                  width: width,
                  height: height
                });

              case 5:
                texture = _context3.sent;
                return _context3.abrupt("return", texture);

              case 7:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function loadImage(_x5) {
        return _loadImage.apply(this, arguments);
      }

      return loadImage;
    }()
  }]);

  return SphereModelBuilder;
}();

__decorate([inject(gPluginWebglRenderer.ShaderModuleService), __metadata("design:type", Object)], SphereModelBuilder.prototype, "shaderModule", void 0);

__decorate([inject(gPluginWebglRenderer.TexturePool), __metadata("design:type", gPluginWebglRenderer.TexturePool)], SphereModelBuilder.prototype, "texturePool", void 0);

SphereModelBuilder = __decorate([injectable()], SphereModelBuilder);

var gridVertex = "attribute vec3 a_Position;\n\nvarying vec3 v_Position;\n\nuniform mat4 u_ProjectionMatrix;\nuniform mat4 u_ViewMatrix;\n\n#pragma include \"instancing.declaration\"\n\nvoid main() {\n  v_Position = a_Position;\n\n  #pragma include \"instancing\"\n\n  gl_Position = u_ProjectionMatrix * u_ViewMatrix * modelMatrix * vec4(a_Position, 1.0);\n}";

var gridFragment = "uniform float u_Opacity : 1;\n\n// generate grid, borrow from clay.gl viewer\n// @see https://github.com/pissang/clay-viewer/blob/master/src/graphic/ground.glsl\n#extension GL_OES_standard_derivatives : enable\n\nvarying vec3 v_Position;\n\nuniform float u_GridSize : 5;\nuniform float u_GridSize2 : .5;\nuniform vec4 u_GridColor : [0, 0, 0, 1];\nuniform vec4 u_GridColor2 : [0.3, 0.3, 0.3, 1];\n\nvoid main() {\n  gl_FragColor = vec4(1.);\n\n  float wx = v_Position.x;\n  float wz = v_Position.z;\n\n  float x1 = abs(fract(wx / u_GridSize2 - 0.5) - 0.5) / fwidth(wx) * u_GridSize2;\n  float z1 = abs(fract(wz / u_GridSize2 - 0.5) - 0.5) / fwidth(wz) * u_GridSize2;\n\n  float v1 = 1.0 - clamp(min(x1, z1), 0.0, 1.0);\n  gl_FragColor = mix(gl_FragColor, u_GridColor2, v1);\n\n  gl_FragColor.a = gl_FragColor.a * u_Opacity;\n}";

var ATTRIBUTE$2 = {
  Position: 'a_Position'
};
var UNIFORM$2 = {
  Color1: 'u_GridColor',
  Color2: 'u_GridColor2',
  Size1: 'u_GridSize',
  Size2: 'u_GridSize2'
};

var GridModelBuilder = /*#__PURE__*/function () {
  function GridModelBuilder() {
    _classCallCheck(this, GridModelBuilder);

    this.shaderModule = void 0;
  }

  _createClass(GridModelBuilder, [{
    key: "onAttributeChanged",
    value: function () {
      var _onAttributeChanged = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(object, name, value) {
        var entity, renderable, renderable3d;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                entity = object.getEntity();
                renderable = entity.getComponent(g.Renderable);
                renderable3d = entity.getComponent(gPluginWebglRenderer.Renderable3D);

              case 3:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      function onAttributeChanged(_x, _x2, _x3) {
        return _onAttributeChanged.apply(this, arguments);
      }

      return onAttributeChanged;
    }()
  }, {
    key: "prepareModel",
    value: function () {
      var _prepareModel = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(object) {
        var _objectSpread2$1;

        var entity, material, geometry, renderable3d, _object$attributes, _object$attributes$fi, fill, height, width, _this$shaderModule$ge, vs, fs, extractedUniforms, fillColor, halfWidth, halfHeight;

        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                entity = object.getEntity();
                material = entity.getComponent(gPluginWebglRenderer.Material3D);
                geometry = entity.getComponent(gPluginWebglRenderer.Geometry3D);
                renderable3d = entity.getComponent(gPluginWebglRenderer.Renderable3D);
                _object$attributes = object.attributes, _object$attributes$fi = _object$attributes.fill, fill = _object$attributes$fi === void 0 ? '' : _object$attributes$fi, height = _object$attributes.height, width = _object$attributes.width;
                this.shaderModule.registerModule('grid', {
                  vs: gridVertex,
                  fs: gridFragment
                });
                _this$shaderModule$ge = this.shaderModule.getModule('grid'), vs = _this$shaderModule$ge.vs, fs = _this$shaderModule$ge.fs, extractedUniforms = _this$shaderModule$ge.uniforms;
                material.vertexShaderGLSL = vs || '';
                material.fragmentShaderGLSL = fs || '';
                material.cull = {
                  enable: false
                };
                material.depth = {
                  enable: false
                };
                fillColor = gPluginWebglRenderer.rgb2arr(fill);
                material.setUniform(_objectSpread2(_objectSpread2({}, extractedUniforms), {}, (_objectSpread2$1 = {}, _defineProperty(_objectSpread2$1, UNIFORM$2.Color1, fillColor), _defineProperty(_objectSpread2$1, UNIFORM$2.Color2, fillColor), _defineProperty(_objectSpread2$1, UNIFORM$2.Size1, 10), _defineProperty(_objectSpread2$1, UNIFORM$2.Size2, 10), _objectSpread2$1)));
                geometry.vertexCount = 6;
                geometry.setIndex([0, 3, 2, 2, 1, 0]);
                halfWidth = width / 2;
                halfHeight = height / 2;
                geometry.setAttribute(ATTRIBUTE$2.Position, Float32Array.from([-halfWidth, 0, -halfHeight, halfWidth, 0, -halfHeight, halfWidth, 0, halfHeight, -halfWidth, 0, halfHeight]), {
                  arrayStride: 4 * 3,
                  stepMode: 'vertex',
                  attributes: [{
                    shaderLocation: 0,
                    offset: 0,
                    format: 'float3'
                  }]
                });

              case 18:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function prepareModel(_x4) {
        return _prepareModel.apply(this, arguments);
      }

      return prepareModel;
    }()
  }]);

  return GridModelBuilder;
}();

__decorate([inject(gPluginWebglRenderer.ShaderModuleService), __metadata("design:type", Object)], GridModelBuilder.prototype, "shaderModule", void 0);

GridModelBuilder = __decorate([injectable()], GridModelBuilder);

var SHAPE_3D;

(function (SHAPE_3D) {
  SHAPE_3D["Cube"] = "cube";
  SHAPE_3D["Sphere"] = "sphere";
  SHAPE_3D["Grid"] = "grid";
})(SHAPE_3D || (SHAPE_3D = {}));

var _excluded = ["style"];
var Cube = /*#__PURE__*/function (_DisplayObject) {
  _inherits(Cube, _DisplayObject);

  var _super = _createSuper(Cube);

  function Cube(_ref) {
    var style = _ref.style,
        rest = _objectWithoutProperties(_ref, _excluded);

    _classCallCheck(this, Cube);

    return _super.call(this, _objectSpread2({
      // @ts-ignore
      type: SHAPE_3D.Cube,
      style: _objectSpread2({
        height: 0,
        width: 0,
        depth: 0,
        widthSegments: 1,
        heightSegments: 1,
        depthSegments: 1
      }, style)
    }, rest));
  }

  return Cube;
}(g.DisplayObject);

var _excluded$1 = ["style"];
var Sphere = /*#__PURE__*/function (_DisplayObject) {
  _inherits(Sphere, _DisplayObject);

  var _super = _createSuper(Sphere);

  function Sphere(_ref) {
    var style = _ref.style,
        rest = _objectWithoutProperties(_ref, _excluded$1);

    _classCallCheck(this, Sphere);

    return _super.call(this, _objectSpread2({
      // @ts-ignore
      type: SHAPE_3D.Sphere,
      style: _objectSpread2({}, style)
    }, rest));
  }

  return Sphere;
}(g.DisplayObject);

var _excluded$2 = ["style"];
var Grid = /*#__PURE__*/function (_DisplayObject) {
  _inherits(Grid, _DisplayObject);

  var _super = _createSuper(Grid);

  function Grid(_ref) {
    var style = _ref.style,
        rest = _objectWithoutProperties(_ref, _excluded$2);

    _classCallCheck(this, Grid);

    return _super.call(this, _objectSpread2({
      // @ts-ignore
      type: SHAPE_3D.Grid,
      style: _objectSpread2({
        height: 0,
        width: 0
      }, style)
    }, rest));
  }

  return Grid;
}(g.DisplayObject);

g.container.bind(g.GeometryAABBUpdater).to(CubeUpdater).inSingletonScope().whenTargetNamed(SHAPE_3D.Cube);
g.container.bind(g.GeometryAABBUpdater).to(SphereUpdater).inSingletonScope().whenTargetNamed(SHAPE_3D.Sphere);
g.container.bind(g.GeometryAABBUpdater).to(GridUpdater).inSingletonScope().whenTargetNamed(SHAPE_3D.Grid);
var containerModule = new ContainerModule(function (bind, unbind, isBound, rebind) {
  gPluginWebglRenderer.registerModelBuilder(CubeModelBuilder, SHAPE_3D.Cube);
  gPluginWebglRenderer.registerModelBuilder(SphereModelBuilder, SHAPE_3D.Sphere);
  gPluginWebglRenderer.registerModelBuilder(GridModelBuilder, SHAPE_3D.Grid);
});

exports.Cube = Cube;
exports.Grid = Grid;
exports.Sphere = Sphere;
exports.containerModule = containerModule;
